@@ -1,4 +1,4 @@
-#===============================================================================
+#=============================================================================
 #
 #       Module:  Term::CLI::L10N
 #
@@ -14,11 +14,11 @@
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 #
-#===============================================================================
+#=============================================================================
 
 use 5.014_001;
 
-package Term::CLI::L10N  0.04009 {
+package Term::CLI::L10N  0.05002 {
 
 use Modern::Perl 1.20140107;
 
@@ -73,7 +73,7 @@ Term::CLI::L10N - localizations for Term::CLI
 
 =head1 VERSION
 
-version 0.04009
+version 0.05002
 
 =head1 SYNOPSIS
 
@@ -90,6 +90,9 @@ version 0.04009
 
 =head1 DESCRIPTION
 
+The C<Term::CLI::L10N> module implements a localization mechanism based
+on L<Locale::Maketext>(3p).
+
 =head1 FUNCTIONS
 
 The module can export a few utility routines.
@@ -127,6 +130,12 @@ Dies with an error if no language can be loaded.
 
 =head1 EXAMPLES
 
+    use Term::CLI::L10N; # Initialise using current locale.
+
+    Term::CLI::L10N->set_language('nl'); # Force "nl" language.
+
+    say loc("ERROR"); # Should print "FOUT".
+
 =head1 SEE ALSO
 
 L<Term::CLI::L10N::en>(3p),
